# READ TRAINING, VALIDATION, AND TEST DATA IN DENSE FORMAT.

read.dense <- function ()
{ 
  train.y <<- scan("train.classes")
  ncases <<- length(train.y)

  train.x <<- read.matrix("train.data.all",nrow=ncases)
  nvars <<- ncol(train.x)

  valid.x <<- read.matrix("valid.data.all",ncol=nvars)
  test.x <<- read.matrix("test.data.all",ncol=nvars)

  invisible()
}

read.matrix <- function (name, nrow=NULL, ncol=NULL)
{
  d <- scan(name)

  if (is.null(nrow))
  { nrow <- length(d)/ncol
  }
  if (is.null(ncol))
  { ncol <- length(d)/nrow
  }

  matrix(d,nrow,ncol,byrow=T)
}


# READ TRAINING, VALIDATION, AND TEST DATA IN SPARSE BINARY FORMAT.

read.sparse.binary <- function (nvars=NULL)
{
  train.y <<- scan("train.classes")
  ncases <<- length(train.y)

  train.x <<- read.binary.list ("train.data.sparse",nvars)
  nvars <<- ncol(train.x) 

  valid.x <<- read.binary.list ("valid.data.sparse",nvars)
  test.x <<- read.binary.list ("test.data.sparse",nvars)

  invisible()
}

read.binary.list <- function (name, nvars=NULL)
{
  d <- scan (pipe (paste ("grep -n .", name, "| sed \"s/\\(.*\\):/-\\1 /\"")))

  if (is.null(nvars))
  { nvars <- max(d)
  }

  nrows <- -min(d)

  r <- double(nrows*nvars)
  dim(r) <- c(nrows,nvars)

  i <- -1

  for (k in 1:length(d))
  { w <- d[k]
    if (w<0)
    { i <- -w
    }
    else
    { r[i,w] <- 1
    }
  }

  r
}


# READ TRAINING, VALIDATION, AND TEST DATA IN SPARSE INTEGER FORMAT.

read.sparse.integer <- function (nvars=NULL)
{
  train.y <<- scan("train.classes")
  ncases <<- length(train.y)

  train.x <<- read.integer.list ("train.data.sparse",nvars)
  nvars <<- ncol(train.x) 

  valid.x <<- read.integer.list ("valid.data.sparse",nvars)
  test.x <<- read.integer.list ("test.data.sparse",nvars)

  invisible()
}

read.integer.list <- function (name, nvars=NULL)
{
  d <- scan (
   pipe (paste ("grep -n .", name, "| sed \"s/\\(.*\\):/-\\1 /\" | tr : \" \""))
  )

  if (is.null(nvars))
  { nvars <- max(d)
  }

  nrows <- -min(d)

  r <- double(nrows*nvars)
  dim(r) <- c(nrows,nvars)

  i <- -1

  k <- 1
  while (k<length(d))
  { w <- d[k]
    if (w<0)
    { i <- -w
      k <- k+1
    }
    else
    { r[i,w] <- d[k+1]
      k <- k+2 
    }
  }

  r
}
